// Bytecode patcher for GLSL shaders.  
// I don't care what you do with this source but I'd like some credit if you reuse this in part or whole. - daxnitro

import javassist.*;
import javassist.expr.*;

import java.io.File;

import java.lang.Exception;

import daxnitro.nitrous.ModHooks;

public class Hooks implements ModHooks {
	public boolean install(File installDir) throws Throwable {

		String dirPath = installDir.getPath();

		ClassPool pool = ClassPool.getDefault();
		pool.appendClassPath(dirPath);
		pool.appendClassPath(new File(new File(Hooks.class.getProtectionDomain().getCodeSource().getLocation().toURI()).getParentFile(), "files").getPath());

		// EntityRenderer.class

		CtClass entityRenderer = pool.get(EntityRenderer);
 
 		CtClass floatParameter[] = {CtClass.floatType};
 		
		CtMethod renderEverything = entityRenderer.getDeclaredMethod(EntityRenderer_renderEverything, floatParameter);

		final Hooks hooks = this;

		renderEverything.instrument(
			new ExprEditor() {
				public void edit(MethodCall m) throws CannotCompileException {
					if (m.getClassName().equals("Shaders") && m.getMethodName().equals("processScene")) {
						hooks.alreadyInstalled = true;
					}
				}
			}
		);
		
		if (alreadyInstalled) {
			System.out.println("Already installed.");
			return true;
		}
		
		renderEverything.instrument(
			new ExprEditor() {
				public void edit(MethodCall m) throws CannotCompileException {
					if (m.getClassName().equals(EntityRenderer) && m.getMethodName().equals(EntityRenderer_renderWorld) && m.getSignature().equals("(F)V")) {
						m.replace("{ $_ = $proceed($$); Shaders.instance.processScene("+EntityRenderer_red+", "+EntityRenderer_green+", "+EntityRenderer_blue+"); }");
					}
				}
			}
		);
		
		CtMethod renderWorld = entityRenderer.getDeclaredMethod(EntityRenderer_renderWorld, floatParameter);

		renderWorld.instrument(
			new ExprEditor() {
				public void edit(NewExpr e) throws CannotCompileException {
					if (e.getClassName().equals(Frustrum)) {
						e.replace("{ Shaders.instance.useProgram(Shaders.instance.baseProgram); $_ = $proceed($$); }");
					}
				}
			}
		);

		renderWorld.instrument(
			new ExprEditor() {
				public void edit(MethodCall m) throws CannotCompileException {
					if (m.getClassName().equals(EntityRenderer) && m.getMethodName().equals(EntityRenderer_renderPlayer) && m.getSignature().equals("(FI)V")) {
						m.replace("{ $_ = $proceed($$); Shaders.copyDepthTexture(Shaders.instance.depthTexture2Id("+EntityRenderer_mc+")); Shaders.instance.useProgram(0); }");
					}
				}
			}
		);

		renderWorld.instrument(
			new ExprEditor() {
				public void edit(MethodCall m) throws CannotCompileException {
					if (m.getClassName().equals(RenderGlobal) && m.getMethodName().equals(RenderGlobal_function) && m.getSignature().equals("(F)V")) {
						m.replace("{ $_ = $proceed($$); Shaders.copyDepthTexture(Shaders.instance.depthTextureId("+EntityRenderer_mc+")); }");
					}
				}
			}
		);

		renderWorld.instrument(
			new ExprEditor() {
				public void edit(MethodCall m) throws CannotCompileException {
					if (m.getClassName().equals(RenderGlobal) && m.getMethodName().equals(RenderGlobal_renderTerrain) && m.getSignature().equals(RenderGlobal_renderTerrain_sig)) {
						m.replace("{ if ($2 == 0) { Shaders.bindTexture(33985, "+EntityRenderer_mc+"."+Minecraft_renderEngine+"."+RenderEngine_getTexture+"(\"/terrain_nh.png\")); Shaders.bindTexture(33986, "+EntityRenderer_mc+"."+Minecraft_renderEngine+"."+RenderEngine_getTexture+"(\"/terrain_s.png\")); Shaders.instance.setRenderType(1); $_ = $proceed($$); Shaders.instance.setRenderType(0); } else { $_ = $proceed($$); } }");
					}
				}
			}
		);

		// Tessellator.class
		
		CtClass tessellator = pool.get(Tessellator);
 
 		CtClass float3Parameter[] = {CtClass.floatType, CtClass.floatType, CtClass.floatType};
 		
		CtMethod setNormal = tessellator.getDeclaredMethod(Tessellator_setNormal, float3Parameter);

		setNormal.insertAfter(
			"byte byte0 = (byte)(int)($1 * 127F);" +
			""+Tessellator_normal+" &= 0xFFFFFF00;" +
			""+Tessellator_normal+" |= byte0;"
			);

 		CtClass double3Parameter[] = {CtClass.doubleType, CtClass.doubleType, CtClass.doubleType};
 		
		CtMethod addVertex = tessellator.getDeclaredMethod(Tessellator_addVertex, double3Parameter);

		addVertex.insertBefore(
			"if ("+Tessellator_drawMode+" == 7 && "+Tessellator_convertQuadsToTriangles+" && ("+Tessellator_addedVertices+" + 1) % 4 == 0 && "+Tessellator_hasNormals+") {" +
			""+Tessellator_rawBuffer+"["+Tessellator_rawBufferIndex+" + 6] = "+Tessellator_rawBuffer+"[("+Tessellator_rawBufferIndex+" - 24) + 6];" +
			""+Tessellator_rawBuffer+"["+Tessellator_rawBufferIndex+" + 8 + 6] = "+Tessellator_rawBuffer+"[("+Tessellator_rawBufferIndex+" + 8 - 16) + 6];" +
			"}"
			);
		
		// RenderBlocks.class
		
		CtClass renderBlocks = pool.get(RenderBlocks);

		CtClass block = pool.get(Block);
		
 		CtClass renderFaceParameter[] = {block, CtClass.doubleType, CtClass.doubleType, CtClass.doubleType, CtClass.intType};
 		
		CtMethod renderBottomFace = renderBlocks.getDeclaredMethod(RenderBlocks_renderBottomFace, renderFaceParameter);		
		renderBottomFace.insertBefore(Tessellator+"."+Tessellator_instance+"."+Tessellator_setNormal+"(0.0F, -1.0F, 0.0F);");

		CtMethod renderTopFace = renderBlocks.getDeclaredMethod(RenderBlocks_renderTopFace, renderFaceParameter);		
		renderTopFace.insertBefore(Tessellator+"."+Tessellator_instance+"."+Tessellator_setNormal+"(0.0F, 1.0F, 0.0F);");

		CtMethod renderEastFace = renderBlocks.getDeclaredMethod(RenderBlocks_renderEastFace, renderFaceParameter);		
		renderEastFace.insertBefore(Tessellator+"."+Tessellator_instance+"."+Tessellator_setNormal+"(0.0F, 0.0F, -1.0F);");

		CtMethod renderWestFace = renderBlocks.getDeclaredMethod(RenderBlocks_renderWestFace, renderFaceParameter);		
		renderWestFace.insertBefore(Tessellator+"."+Tessellator_instance+"."+Tessellator_setNormal+"(0.0F, 0.0F, 1.0F);");

		CtMethod renderNorthFace = renderBlocks.getDeclaredMethod(RenderBlocks_renderNorthFace, renderFaceParameter);		
		renderNorthFace.insertBefore(Tessellator+"."+Tessellator_instance+"."+Tessellator_setNormal+"(-1.0F, 0.0F, 0.0F);");

		CtMethod renderSouthFace = renderBlocks.getDeclaredMethod(RenderBlocks_renderSouthFace, renderFaceParameter);		
		renderSouthFace.insertBefore(Tessellator+"."+Tessellator_instance+"."+Tessellator_setNormal+"(1.0F, 0.0F, 0.0F);");
		
		// RenderEngine.class
		
		CtClass renderEngine = pool.get(RenderEngine);
		
 		CtClass stringParameter[] = {pool.get("java.lang.String")};
 		
		CtMethod getTexture = renderEngine.getDeclaredMethod(RenderEngine_getTexture, stringParameter);

		getTexture.instrument(
			new ExprEditor() {
				public void edit(MethodCall m) throws CannotCompileException {
					if (m.getClassName().equals(TexturePackBase) && m.getMethodName().equals(TexturePackBase_getStream) && m.getSignature().equals("(Ljava/lang/String;)Ljava/io/InputStream;")) {
						m.replace("{ $_ = $proceed($$); if ($_ == null) { return 0; } }");
					}
				}
			}
		);

		entityRenderer.writeFile(dirPath);
		tessellator.writeFile(dirPath);
		renderBlocks.writeFile(dirPath);
		renderEngine.writeFile(dirPath);

		return true;
	}
	
	public boolean alreadyInstalled = false;
	
	// Obfuscated names. Hopefully this is all that will need to be changed for future releases.

	static final String EntityRenderer = "ll";
	static final String EntityRenderer_mc = "h";
	static final String EntityRenderer_renderEverything = "b";
	static final String EntityRenderer_renderWorld = "c";
	static final String EntityRenderer_renderPlayer = "b";
	static final String EntityRenderer_red = "e";
	static final String EntityRenderer_green = "this.f";
	static final String EntityRenderer_blue = "g";
	
	static final String Frustrum = "np";
	
	static final String RenderGlobal = "h";	
	static final String RenderGlobal_function = "b";
	static final String RenderGlobal_renderTerrain = "a";
	static final String RenderGlobal_renderTerrain_sig = "(Leu;ID)I";

	static final String Minecraft_renderEngine = "n";
	
	static final String RenderEngine = "gs";
	static final String RenderEngine_getTexture = "a";
	
	static final String TexturePackBase = "e";
	static final String TexturePackBase_getStream = "a";

	static final String Tessellator = "jy";
	static final String Tessellator_instance = "a";
	static final String Tessellator_setNormal = "b";
	static final String Tessellator_normal = "v";
	static final String Tessellator_addVertex = "a";
	static final String Tessellator_addedVertices = "p";
	static final String Tessellator_drawMode = "r";
	static final String Tessellator_convertQuadsToTriangles = "b";
	static final String Tessellator_hasNormals = "n";
	static final String Tessellator_rawBuffer = "g";
	static final String Tessellator_rawBufferIndex = "o";
	
	static final String RenderBlocks = "bt";
	static final String RenderBlocks_renderBottomFace = "a";
	static final String RenderBlocks_renderTopFace = "b";
	static final String RenderBlocks_renderEastFace = "c";
	static final String RenderBlocks_renderWestFace = "d";
	static final String RenderBlocks_renderNorthFace = "e";
	static final String RenderBlocks_renderSouthFace = "f";

	static final String Block = "pj";
	
}